# schm

[![NPM version](https://img.shields.io/npm/v/schm.svg?style=flat-square)](https://npmjs.org/package/schm)

## Install

    $ npm install --save schm

## Usage

```js
const schema = require('schm')

const userSchema = schema({
  name: String,
  age: Number,
  height: Number,
  skills: [RegExp],
})

const user = userSchema.parse({ 
  name: 'Haz', 
  age: '27', 
  height: '1.88', 
  skills: ['code', 'design', 'astronomy'],
})
```

Output:

```js
{
  name: 'Haz',
  age: 27,
  height: 1.88,
  skills: [/code/i, /design/i, /astronomy/i],
}
```

The way you declare the schema object is very similar to [mongoose Schemas](http://mongoosejs.com/docs/guide.html). So, refer to their docs to learn more.

### Extending schemas

You can extend the functionality of schemas by composing them. That way, you can add custom parsers and validators or modify the behavior of [`parse`](#parse) and [`validate`](#validate) methods themselves. See [other packages](../)'s source code to learn more. Here's a simple example:

```js
const schema = require('schm')

const userSchema = schema({
  name: String,
  age: {
    type: Number,
    adult: true,
  },
}, previous => previous.merge({
  validators: {
    adult: value => ({
      valid: value >= 18,
      message: 'Not adult',
    }),
  },
}))

userSchema.validate({ name: 'John', age: 17 }) // error
```

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

-   [schema](#schema)
    -   [Parameters](#parameters)
    -   [Examples](#examples)
-   [parse](#parse)
    -   [Parameters](#parameters-1)
    -   [Examples](#examples-1)
-   [validate](#validate)
    -   [Parameters](#parameters-2)
    -   [Examples](#examples-2)
-   [group](#group)
    -   [Parameters](#parameters-3)
    -   [Examples](#examples-3)
-   [Types](#types)
    -   [Schema](#schema-1)
        -   [Properties](#properties)
    -   [SchemaGroup](#schemagroup)
    -   [ValidationError](#validationerror)
        -   [Properties](#properties-1)

### schema

Creates a schema by composing groups of parameters.

#### Parameters

-   `groups` **...[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;([Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object) \| [Schema](#schema) \| [SchemaGroup](#schemagroup))>** 

#### Examples

```javascript
const schema = require('schm')

const userSchema = schema({
  id: String,
  name: String,
}, {
  age: Number
})

// nested schema
const teamSchema = schema({
  users: [userSchema],
})
```

Returns **[Schema](#schema)** 

### parse

Parses a schema based on given values.

#### Parameters

-   `values` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)**  (optional, default `{}`)
-   `params` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 

#### Examples

```javascript
const { parse } = require('schm')

parse(
  { foo: 1, bar: '1' },
  { foo: String, bar: Number },
)
// -> { foo: '1', bar: 1 }

// can also be used directly from schema
schema({ foo: String, bar: Number }).parse({ foo: 1, bar: '1' })
```

Returns **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 

### validate

Validates a schema based on given values.

#### Parameters

-   `values` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)**  (optional, default `{}`)
-   `params` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 
-   `paramPathPrefix` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### Examples

```javascript
const schema = require('schm')
const { validate } = schema

const userSchema = schema({
  name: {
    type: String,
    required: true,
  },
  age: {
    type: Number,
    min: [18, 'Too young'],
  }
})

validate({ name: 'John', age: 17 }, userSchema)
  .then((parsedValues) => {
    console.log('Yaay!', parsedValues)
  })
  .catch((errors) => {
    console.log('Oops!', errors)
  })

// Output:
// Oops! [{
//   param: 'age',
//   value: 17,
//   validator: 'min',
//   min: 18,
//   message: 'Too young',
// }]
```

Returns **[Promise](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[ValidationError](#validationerror)>>** 

### group

A simple group of parameters. It's used internally when you pass literal
objects to [`schema`](#schema).

#### Parameters

-   `params` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)**  (optional, default `{}`)

#### Examples

```javascript
const schema = require('schm')
const { group } = schema

const userSchema = schema(
  group({
    id: String,
    name: String,
  }),
  group({
    age: Number,
  })
)
```

Returns **[SchemaGroup](#schemagroup)** 

### Types




#### Schema

Type: {params: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object), parsers: {}, validators: {}, parse: function (values: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)): [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object), validate: function (values: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)): [Promise](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[ValidationError](#validationerror)>>, merge: function (): [Schema](#schema)}

##### Properties

-   `params` **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 
-   `parsers` **{}** 
-   `validators` **{}** 
-   `parse` **function (values: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)): [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 
-   `validate` **function (values: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)): [Promise](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Promise)&lt;[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[ValidationError](#validationerror)>>** 
-   `merge` **function (): [Schema](#schema)** 

#### SchemaGroup

Type: function (previous: [Schema](#schema)): [Schema](#schema)

#### ValidationError

Type: {param: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String), value: any?, validator: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String), message: [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?}

##### Properties

-   `param` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `value` **any?** 
-   `validator` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `message` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** 

## License

MIT © [Diego Haz](https://github.com/diegohaz)
